<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * Templater plugin
 * -------------------------------------------------------------
 * Type:     function
 * Name:     xoffers_promo
 * Purpose:  generate list of random promo blocks
 * -------------------------------------------------------------
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage Lib
 * @author     Ruslan R. Fazlyev <rrf@x-cart.com>
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    9420d43903cbe798e4b468f13709aaacf26ac8a2, v28 (xcart_4_7_9), 2018-01-09 16:56:31, function.xoffers_promo.php, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */

if (!defined('XCART_START')) { header("Location: ../../"); die("Access denied"); }

function smarty_function_xoffers_promo($params, &$smarty) {//{{{
    global $logged_userid, $login_type;
    $check_vars = array('product_offers', 'category_offers');

    if (empty($params['mode']) || $params['mode'] != 'random')
        return;

    foreach ($check_vars as $v) {
        $_v = $smarty->getTemplateVars($v);
        if (!empty($_v))
            return;
    }

    $avail_offers = func_get_offers($logged_userid, $login_type, false);

    if (empty($avail_offers)) {
        return;
    }

    $avail_offers = func_offer_unset_not_applicable4product_page($avail_offers);
    $avail_offers = func_offer_unset_not_applicable4category_page($avail_offers);

    $offers_with_short_promo = array();
    foreach ($avail_offers as $v) {
        if ($v['show_short_promo'] == 'no') {
            continue;
        }

        $offers_with_short_promo[] = $v['offerid'];
    }

    if (empty($offers_with_short_promo)) {
        return;
    }

    shuffle($offers_with_short_promo);

    $info_offers = func_get_sorted_offers($offers_with_short_promo);
    $smarty->assign('new_offers', $info_offers);
    $template = 'modules/Special_Offers/customer/new_offers_short_list.tpl';
    $smarty->smarty->already_included_tpls[$template] = ++$smarty->smarty->inclusion_depth;
    $smarty->display($template);
    $smarty->smarty->inclusion_depth--;
}//SMARTy_FUNCTion_xoffers_promo}}}

function func_offer_unset_not_applicable4product_page($offers) {//{{{
    global $smarty;

    if (empty($offers)) {
        return $offers;
    }

    foreach ($offers as $offer_ind => $_offer) {
        if (
            empty($_offer['bonuses'])
            || count($_offer['bonuses']) > 1
            || empty($_offer['bonuses'][0]['product_sets']['E'])
            || $_offer['bonuses'][0]['bonus_type'] != 'D'
        ) {
            // Unset short promo only for offers where 'Give a discount' and "Allow special offers disconts" is disabled
            continue;
        }

        $current_product = isset($current_product) ? $current_product : $smarty->getTemplateVars('product');

        if (
            empty($current_product)
            || empty($current_product['sp_discount_avail'])
            || $current_product['sp_discount_avail'] != 'N'
        ) {
            return $offers;
        }

        $current_productid = $current_product['productid'];
        foreach ($_offer['bonuses'][0]['params'] as $bonus_param) {
            if (
                $bonus_param['param_id'] == $current_productid
                && $bonus_param['param_type'] == 'P'
                && in_array($bonus_param['setid'], $_offer['bonuses'][0]['product_sets']['E'])
            ) {
                // the current product has Sp_discount_avail = N
                unset($offers[$offer_ind]);
                break;
            }
        }
    }

    return $offers;
}//}}}

function func_offer_unset_not_applicable4category_page($offers) {//{{{
    global $smarty, $sql_tbl;

    if (empty($offers)) {
        return $offers;
    }

    foreach ($offers as $offer_ind => $_offer) {
        if (
            $_offer['show_short_promo'] != 'yes_on_conditional_pages'
            || empty($_offer['conditions'])
            || count($_offer['conditions']) > 1
            || empty($_offer['conditions'][0]['product_sets']['I'])
            || $_offer['conditions'][0]['condition_type'] != 'S'
        ) {
            // Unset short promo only for offers where Condition is 'Customer orders a certain product or products' and set is 'Products from category: Puzzles'
            continue;
        }

        $has_product_param = false;
        foreach ($_offer['conditions'][0]['params'] as $condition_param) {
            if ($condition_param['param_type'] != 'C') {
                $has_product_param = true;
                break;
            }
        }

        if (!empty($has_product_param)) {
            // if has any product condition then show promo anyway
            continue;
        }

        $current_category = isset($current_category) ? $current_category : $smarty->getTemplateVars('current_category');

        if (empty($current_category) || empty($current_category['categoryid'])) {
            // the current page is not conditional category page so must be Unset
            unset($offers[$offer_ind]);
            continue;
        }

        $is_exact_conditional_cat = false;
        $parent_catids = array();
        foreach ($_offer['conditions'][0]['params'] as $condition_param) {
            if ($current_category['categoryid'] == $condition_param['param_id']) {
                $is_exact_conditional_cat = true;
                break;
            }

            if ($condition_param['param_arg'] == 'Y') {
                $parent_catids[] = $condition_param['param_id'];
            }
        }

        if (
            !empty($is_exact_conditional_cat) // the current page is conditional category page - show promo for it
            || empty($current_category['lpos']) // For some reason no data for the category - show promo for it
            || empty($current_category['rpos'])
        ) {
            // the current page is conditional category page so mustn't to be Unset
            continue;
        }

        if (
            empty($parent_catids) // the current category is not conditional category page so must be Unset. 'including subcategories' is disabled
            || (isset($current_category['parentid']) && $current_category['parentid'] == 0) // the current category is not conditional category page so must be Unset. There are no parents for the category
        ) {
            unset($offers[$offer_ind]);
            continue;
        }

        // try to search the current category in subcats
        $current_cat_is_child = func_query_first_cell("SELECT COUNT(categoryid) FROM $sql_tbl[categories] WHERE lpos<'$current_category[lpos]' AND rpos>'$current_category[rpos]' AND categoryid IN ('".implode("','", $parent_catids)."')");
        if (!$current_cat_is_child) {
            // the current category is not under conditional categories page so must be Unset. 'including subcategories' is enabled
            unset($offers[$offer_ind]);
            continue;
        }
    }

    return $offers;
}//}}}
