<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * Import/export amazon product type
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage Modules
 * @author     Ildar Amankulov
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    fd70535d7498b633d97405233cf44cda7b81ff8f, v1 (xcart_4_7_10), 2018-02-01 12:37:00, import.php, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */

/******************************************************************************
Used cache format:
Products (by Product ID):
    data_type:  PI
    key:        <Product ID>
    value:      [<Product code> | RESERVED]
Products (by Product code):
    data_type:  PR
    key:        <Product code>
    value:      [<Product ID> | RESERVED]
Products (by Product name):
    data_type:  PN
    key:        <Product name>
    value:      [<Product ID> | RESERVED]
Deleted product data:
    data_type:  DP
    key:        <Product ID>
    value:      <Flags>

Note: RESERVED is used if ID is unknown
******************************************************************************/

if ( !defined('XCART_SESSION_START') ) { header("Location: ../"); die("Access denied"); }

if ($import_step == 'process_row') {
    /**
     * PROCESS ROW from import file
     */

    // Check productid / productcode / product
    list($_productid, $_variantid) = func_import_detect_product($values);
    if (is_null($_productid) || ($action == 'do' && empty($_productid))) {
        func_import_module_error('msg_err_import_log_message_14');
        return false;
    }

    if (empty($g_amazon_cats)) {
        func_amazon_feeds_load_classes();
        $g_amazon_cats = \XCart\Modules\AmazonFeeds\Feeds\Catalog\Categories::getInstance()->getFlattenCategories();
        $g_amazon_cats = call_user_func_array('array_merge', $g_amazon_cats);
    }

    if (!empty($values['product_type']) && !isset($g_amazon_cats[$values['product_type']])) {
        func_import_module_error('txt_amazon_feeds_import_unknown_cat', array('name'=>$values['product_type']));
        return false;
    }

    $values['productid'] = $_productid;

    $data_row[] = $values;

} elseif ($import_step == 'finalize') {
    /**
     * FINALIZE rows processing: update database
     */

    if (!empty($import_file['drop'][strtolower($section)]) && $import_file['drop'][strtolower($section)] == 'Y') { //{{{
        if (!empty($provider_condition)) {
            // Search for products created by provider...
            db_query("DELETE FROM $sql_tbl[amazon_feeds_catalog] WHERE productid IN (SELECT productid FROM $sql_tbl[products] WHERE 1 $provider_condition)");
        } else {
            db_query("TRUNCATE TABLE $sql_tbl[amazon_feeds_catalog]");
        }

        $import_file['drop'][strtolower($section)] = '';
    } //}}}

    foreach ($data_row as $row) {
        $data = array_intersect_key($row, $import_specification[$section]['columns']);
        func_unset($data, 'productcode', 'product');
        $data = func_addslashes($data);

        if (empty($data['product_type'])) {
            db_query("DELETE FROM $sql_tbl[amazon_feeds_catalog] WHERE productid = '$data[productid]'");
            if (db_affected_rows()) {
                $result[strtolower($section)]['updated']++;
            }
            continue;
        }

        $productid = false;
        if (!empty($data['productid'])) {
            $productid = func_query_first_cell("SELECT productid FROM $sql_tbl[amazon_feeds_catalog] WHERE productid = '$data[productid]'");
        }

        if (!empty($productid)) {
            func_array2update('amazon_feeds_catalog', $data, "productid='" . $productid . "'");
            $result[strtolower($section)]['updated']++;
        } else {
            func_array2insert('amazon_feeds_catalog', $data);
            $result[strtolower($section)]['added']++;
        }

        echo ". ";
        func_flush();

    }

} elseif ($import_step == 'export') {
    // Export data

    while ($productid = func_export_get_row($data)) {
        if (empty($productid)) {
            continue;
        }

        // Get data
        if (empty($provider_sql)) {
            $row = func_query_first("SELECT * FROM $sql_tbl[amazon_feeds_catalog] WHERE productid = '$productid'");
        } else {
            $row = func_query_first("SELECT afc.* FROM $sql_tbl[amazon_feeds_catalog] afc INNER JOIN $sql_tbl[products] p ON p.productid=afc.productid AND p.provider = '$provider_sql' WHERE afc.productid = '$productid'");
        }

        if (empty($row)) {
            continue;
        }

        if (!func_export_write_row($row)) {
            break;
        }
    }
}
