<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * TaxJar Integration Module
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage TaxJar
 * @author     Qualiteam software Ltd <info@x-cart.com>
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    108b66d4a1516cb3fa5b36f93bf695f6dee9de1e, v5 (xcart_4_7_12), 2020-03-10 18:05:51, func.php, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */
if (!defined('XCART_START')) { header('Location: ../../'); die('Access denied'); }

define('XC_TAXJAR_TAX_NAME', 'TAXJAR_TAX');

class XC_TaxJar {
    public static function getProductUniqId($_product) {//{{{
        return $_product['productid']
            . (empty($_product['variantid']) ? '' : "-$_product[variantid]")
            . (empty($_product['price']) || floatval($_product['price']) == 0 ? '-free' : '')
        ;
    }//}}}

    public static function setExemptionType($_params, $_customer_info) {//{{{
        global $config;

        $customerIsExempt = !empty($_customer_info['tax_exempt'])
            && $_customer_info['tax_exempt'] == 'Y'
            && $config['Taxes']['tax_operation_scheme'] == XCTaxesDefs::TAX_SCHEME_GENERAL;

        if ($customerIsExempt) {
            $_params['exemption_type'] = 'other';
        }
        return $_params;
    }//}}}

    public static function setProductTaxCode($_param, $_product) {//{{{
        if (
            !empty($_product['csi_taxjar_code'])
            && $_product['csi_taxjar_code'] != '00000'
        ) {
            $_param['product_tax_code'] = $_product['csi_taxjar_code'];
        }

        return $_param;
    }//}}}
}

/**
 * Get module config
 * @return array
 */
function func_taxjar_Config()
{ //{{{
    global $config;

    return $config['CSI_TaxJar'];
} //}}}

/**
 * Log events
 * @param object|array $e Error message object or array
 *
 * @return void
 */
function func_taxjar_LogEvent($e)
{ //{{{
    $msg = 'Error at ' . date('Y-m-d H:i:s', constant('XC_TIME'))
        . '; Error code - ' . (is_object($e) ? $e->getStatusCode() : $e['response']['code'])
        . '; Error message - ' . (is_object($e) ? $e->getMessage() : $e['response']['msg']);

    x_log_add('taxjar', $msg, true);
} //}}}

/**
 * Check if module is set
 * @return boolean
 */
function func_taxjar_IsSet()
{ //{{{
    $config = func_taxjar_Config();

    return (!empty($config['taxjar_api_live']) && 'live' == $config['taxjar_env'])
        || (!empty($config['taxjar_api_test']) && 'test' == $config['taxjar_env']);
} //}}}

/**
 * Initialize Module
 * @return void
 */
function func_taxjar_module_init()
{ //{{{
    global $smarty;

    if (defined('ADMIN_MODULES_CONTROLLER')) { // Compatible Added In Xcart_4_5_3
        func_add_event_listener('module.ajax.toggle', 'func_taxjar_Toggle');
    }

    if (defined('ADMIN_CONFIGURATION_CONTROLLER')) { // Compatible Added In Xcart_4_6_6
        func_add_event_listener('evt-module-setting-before-display4712-taxjar_ship_local', 'func_taxjar_admin_change_taxjar_ship_local');
    }

    // Compatible Added In Xcart_4_7_10
    func_add_event_listener('evt_order_status_processed', function($arr, $arr2, $called_from_func_taxjar_module_init = '') {func_taxjar_CreateUpdateOrder($arr2['order_data']);});#nolint
    func_add_event_listener('evt_order_status_completed', function($arr, $arr2, $called_from_func_taxjar_module_init = '') {func_taxjar_CreateUpdateOrder($arr2['order_data']);});#nolint
    func_add_event_listener('evt_order_status_refunded', function($arr, $arr2, $called_from_func_taxjar_module_init = '') {func_taxjar_ShowDeleteOrder('delete', $arr['orderid']);});#nolint
    func_add_event_listener('evt_order_status_declined', function($arr, $arr2, $called_from_func_taxjar_module_init = '') {func_taxjar_ShowDeleteOrder('delete', $arr['orderid']);});#nolint

    func_add_event_listener('evt-returns-list-delete4712', function($arr_to_del = array(), $called_from_returns_php_delete = '') {foreach ($arr_to_del as $_id) func_taxjar_ShowDeleteRefund('delete', $_id);});#nolint

    $area = (defined('AREA_TYPE')) ? AREA_TYPE : '';

    if (defined('QUICK_START') || $area == 'C') {
        return;
    }


    // Smarty_prefilter_add_csi_taxjar_entry_points is called here
    $smarty->addAutoloadFilters(array('add_csi_taxjar_entry_points'), 'pre');
} //}}}

/*
* Dynamically Add {Include File="Modules/CSI_TaxJar/* Calls To Customer Tpls
*/
function Smarty_prefilter_add_csi_taxjar_entry_points($source, $smarty)
{//{{{
    global $xcart_dir;
    static $is_called ;

    if (!empty($is_called)) {
        return $source;
    }
    $is_called = 1;

    require_once $xcart_dir . XC_DS . 'include' . XC_DS . 'lib' . XC_DS . 'dynamic_tpl_patcher.php';
    x_tpl_prefilter_append_to_file('main/product_details.tpl', array('file' => 'skin/common_files/modules/CSI_TaxJar/product_modify.tpl', 'insert_before' => '{if $active_modules.AvaTax' , 'module_name' => 'CSI_TaxJar'));#nolint

    return $source;
}//}}}

/**
 * TaxJar cannot be used together with other tax modules
 * @param string  $module_name Module name
 * @param boolean $active      If module is active or not
 *
 * @return string
 */
function func_taxjar_Toggle($module_name, $active, $previous_handler_result)
{ //{{{
    $return = $previous_handler_result;

    if ($active) {
        global $sql_tbl, $active_modules, $smarty;

        $modules = ['CSI_TaxJar', 'AvaTax', 'TaxCloud'];

        if (in_array($module_name, $modules)) {
            foreach ($modules as $module) {
                if ($module != $module_name && !empty($active_modules[$module])) {
                    db_query("UPDATE $sql_tbl[modules] SET active = 'N' WHERE module_name = '$module'");

                    $return = 'modules.php';
                }
            }

            $active_modules = func_get_active_modules(true);

            $smarty->assign_by_ref('active_modules', $active_modules);
        }
    }

    return $return;
} //}}}

/**
 * Add Variants For The Setting
 */
function func_taxjar_admin_change_taxjar_ship_local($option, $_var, $prev_result)
{ //{{{
    global $sql_tbl;
    static $__shipping;

    if ($option == 'CSI_TaxJar') {
        $__shipping = !empty($__shipping) ? $__shipping : func_query("SELECT shippingid, shipping FROM $sql_tbl[shipping] WHERE code = '' AND active = 'Y' ORDER BY orderby");

        if ($__shipping) {
            if (!is_array($_var['variants'])) {
                $_var['variants'] = array();
            }
            foreach ($__shipping as $ek => $ev) {
                $_var['variants'][$ev['shippingid']] = array('name' => $ev['shipping']);
            }
        }
    }

    return $_var;
} //}}}

/*
 * Generates hash code
 * @return trsing
 */
function func_taxjar_GenerateHash()
{ //{{{
    return md5(serialize(func_get_args()));
} //}}}

/*
 * Get cached value of request
 * @return array
 */
function func_taxjar_Cache($func, $args)
{ //{{{
    static $cache = [];

    $cacheKey = func_taxjar_GenerateHash($func, $args);

    if (!isset($cache[$cacheKey])) {
        $cachedValue = func_taxjar_CacheGet($cacheKey);

        if (is_null($cachedValue)) {
            $cachedValue = call_user_func_array($func, $args);

            func_taxjar_CacheSet($cacheKey, $cachedValue);
        }

        $cache[$cacheKey] = $cachedValue;
    }

    return $cache[$cacheKey];
} //}}}

/*
 * Get cache from database
 *
 * @return array
 */
function func_taxjar_CacheGet($key)
{ //{{{
    global $sql_tbl;
    static $res = array();

    if (isset($res[$key])) {
        return $res[$key];
    }

    $value = func_query_first_cell("SELECT value FROM $sql_tbl[taxjar_cache] WHERE cache_key = '$key' AND expr > " . constant('XC_TIME'));

    if ($value !== false) {
        $res[$key] = !empty($value) ? unserialize($value) : $value;
    } else {
        $res[$key] = null;
    }
    return $res[$key];
} //}}}

/*
 * Save cache in database for later use
 *
 * @return void
 */
function func_taxjar_CacheSet($key, $value)
{ //{{{
    $data = [
        'cache_key' => $key,
        'value'     => serialize($value),
        'expr'      => constant('XC_TIME') + constant('XC_TAXJAR_REQUEST_CACHE_TTL') * 60,
    ];

    !mt_rand(0, 20) && func_taxjar_CacheRemove(constant('XC_TIME'));

    func_array2insert('taxjar_cache', $data, true);
} //}}}

/*
 * Remove cache from database
 *
 * @return void
 */
function func_taxjar_CacheRemove($time = '')
{ //{{{
    global $sql_tbl;
    $where = empty($time) ? 1 : ('expr < ' . $time);

    db_query("DELETE FROM $sql_tbl[taxjar_cache] WHERE $where");
} //}}}

/**
 * Initialize TaxJar connection
 *
 * @return TaxJar
 */
function func_taxjar_Client()
{ //{{{
    global $xcart_dir;
    $client = null;

    if (func_taxjar_IsSet()) {

        // Load TaxJar and Guzzle API
        require_once $xcart_dir . XC_DS . 'modules' . XC_DS . 'CSI_TaxJar' . '/lib/vendor/autoload.php';

        $config = func_taxjar_Config();

        if ('test' == $config['taxjar_env']) {
            $client = \TaxJar\Client::withApiKey($config['taxjar_api_test']);
            $client->setApiConfig('api_url', TaxJar\Client::SANDBOX_API_URL);
        } else {
            $client = \TaxJar\Client::withApiKey($config['taxjar_api_live']);
        }
    }

    return $client;
} //}}}

/**
 * Initialize TaxJar connection and get response
 *
 * @param string $action Action to perform
 * @param mixed  $params Parameters for action (array or string)
 *
 * @return array
 */
function func_taxjar_SendRequest($action, $params = '')
{ //{{{
    global $top_message;
    static $count = 0;

    $config = func_taxjar_Config();

    $result = [
        'error'     => true,
        'response'  => [
            'code'  => '422',
            'msg'   => 'TaxJar module is not setup or active.'
        ]
    ];

    if (in_array($action, array('taxForOrder', 'validate', 'nexusRegions', 'summaryRates'))) {
        $_cache_key = md5(serialize(array($action, $params, $config)));
        $val = func_taxjar_CacheGet($_cache_key);

        if (!is_null($val)) {
            return $val;
        }
    }

    $client = func_taxjar_Client();

    if ($client) {
        try {
            $result['error'] = false;

            if (in_array($action, ['taxForOrder'])) {
                $params['plugin'] = 'xcart';
            }


            if (defined('XC_TAXJAR_DEBUG_CACHE')) {
                x_log_add('taxjar_debug_cache' . XC_TIME . '_' . $count, $action . print_r($params, true));
                $count++;
            }

            if ('ratesForLocation' == $action) {
                $zipcode = $params['zipcode'];
                unset($params['zipcode']);

                $result['response'] = json_decode(json_encode($client->{$action}($zipcode, $params)), true);
            } else {
                $result['response'] = json_decode(json_encode($client->{$action}($params)), true);
            }
        } catch (TaxJar\Exception $e) {
            $result = [
                'error'     => true,
                'response'  => [
                    'code'  => $e->getStatusCode(),
                    'msg'   => $e->getMessage()
                ]
            ];
        } catch (\Exception $e) {
            $result = [
                'error'     => true,
                'response'  => [
                    'code'  => $e->getCode(),
                    'msg'   => $e->getMessage()
                ]
            ];
        }
    } else {
        // Disable Cache
        $_cache_key = '';
    }

    if ('Y' == $config['taxjar_debug']) {
        $debugInfo = [
            'action'    => $action,
            'params'    => $params,
            'result'    => $result,
        ];

        x_log_add('taxjar_debugs', $debugInfo, true);
    }

    if ($result['error']) {
        func_taxjar_LogEvent($result);

        $top_message['type'] = 'E';
        $top_message['content'] = $result['response']['msg'];
    }

    if (!empty($_cache_key)) {
        func_taxjar_CacheSet($_cache_key, $result);
    }

    return $result;
} //}}}

/**
 * Get date range
 *
 * @param mixed $dateRange From-to date or specific date as Unix timestamp
 *
 * @return array|string
 */
function func_taxjar_DateRange($dateRange)
{ //{{{
    $params = '';

    if (null !== $dateRange) {
        if (!is_array($dateRange)) {
            $dateRange = [$dateRange];
        }

        $fromDate = $toDate = time();

        if (
            isset($dateRange[0])
            && intval($dateRange[0]) > 0
        ) {
            $fromDate = intval($dateRange[0]);
        }

        if (
            isset($dateRange[1])
            && intval($dateRange[1]) > 0
        ) {
            $toDate = intval($dateRange[1]);
        }

        if ($fromDate == $toDate) {
            $params = [
                'transaction_date' => date('Y/m/d', $fromDate),
            ];
        } else {
            $params = [
                'from_transaction_date' => date('Y/m/d', $fromDate),
                'to_transaction_date'   => date('Y/m/d', $toDate),
            ];
        }
    }

    return $params;
} //}}}

/**
 * Get TaxJar categories
 *
 * @param boolean $force Force categories download from TaxJar
 *
 * @return array
 */
function func_taxjar_Categories($force = false)
{ //{{{
    global $xcart_dir;

    $filePath = $xcart_dir . XC_DS . 'modules' . XC_DS . 'CSI_TaxJar' . XC_DS . 'categories.json';

    $categories = $force ? null : func_taxjar_CategoriesFromFile($filePath);

    if (empty($categories)) {
        $response = func_taxjar_SendRequest('categories');

        if (!$response['error']) {
            $file = fopen($filePath, 'w');

            if (false !== $file) {

                if (is_array($response['response'])) {
                    $response['response'][] = array(
                        "product_tax_code" =>  "99999",
                        "name" =>  "Other Exempt",
                        "description" =>  "Item is exempt",
                    );
                }
                fwrite($file, json_encode($response['response']));
            }

            fclose($file);

            $categories = func_taxjar_CategoriesFromFile($filePath);
        }
    }

    usort($categories, function ($a, $b) {#nolint
        return strcmp($a['name'], $b['name']);#nolint
    });

    return $categories ?: [];
} //}}}

/**
 * Get TaxJar categories from local file
 *
 * @param string $filePath File path
 *
 * @return array
 */
function func_taxjar_CategoriesFromFile($filePath)
{ //{{{
    return is_readable($filePath)
        ? json_decode(file_get_contents($filePath), true)
        : null;
} //}}}

/**
 * Check if need to calculate tax based on US nexus
 *
 * @param array $CustomerInfo Customer information
 *
 * @return boolean
 */
function func_taxjar_HasUsaNexus($customerInfo)
{ //{{{
    return 'US' === $customerInfo['s_country']
        && in_array($customerInfo['s_state'], func_taxjar_NexusStates());
} //}}}

/**
 * Get USA Nexus States From Config
 *
 * @return array
 */
function func_taxjar_NexusStates()
{ //{{{
    $config = func_taxjar_Config();

    return empty($config['taxjar_usa_nexus']) ? array() : array_map('trim', preg_split('/[' . PHP_EOL . ',;]/Ss', $config['taxjar_usa_nexus']));
} //}}}

/**
 * Get nexus regions
 * List existing nexus locations fron TaxJar account
 *
 * @return array
 */
function func_taxjar_NexusRegions()
{ //{{{
    return func_taxjar_SendRequest('nexusRegions');
} //}}}

/**
 * Main Calculate Function
 */
function func_taxjar_get_taxes_n_products($products, $customer_info, $shipping_cost)
{//{{{
    $taxes = func_taxjar_TaxForOrder($products, $customer_info, $shipping_cost);

    if (!$taxes) {
        return array(false, $products);
    }

    $taxJarResponse = isset($taxes['taxjar_response']) ? unserialize($taxes['taxjar_response']) : false;

    if ($taxJarResponse && isset($taxJarResponse['breakdown'])) {
        $taxJarProducts = $taxJarResponse['breakdown']['line_items'];

        foreach ($products as $key => $product) {
            $products[$key]['taxes'] = [];

            foreach ($taxJarProducts as $item) {
                if ($item['id'] == $product['productcode']) {
                    $products[$key]['taxes'] = [
                        XC_TAXJAR_TAX_NAME => [
                            'tax_name' => XC_TAXJAR_TAX_NAME,
                            'address_type' => 'S',
                            'price_includes_tax' => 'N',
                            'display_including_tax' => 'N',
                            'priority' => 0,
                            'rate_value' => $item['combined_tax_rate'] * 100,
                            'rate_type' => '%',
                            'tax_display_name' => 'Sales Tax',
                            'tax_value_precise' => $item['tax_collectable'],
                            'tax_value' => $item['tax_collectable'],
                            'taxed_price' => $item['taxable_amount'] + $item['tax_collectable']
                        ],
                    ];

                    break;
                }
            }
        }
    }

    return array($taxes, $products);
}//}}}

/**
 * Validate address
 * Use cache to get data
 *
 * @param array  $address Customer address
 *
 * @return array
 */
function func_taxjar_ValidateTaxAddress($address, $result)
{ //{{{
    $config = func_taxjar_Config();

    if (
        'Y' != $config['taxjar_validate_address']
        || empty($address['country']) || $address['country'] != 'US') { // https://developers.taxjar.com/api/reference/#post-validate-an-address At this time only US addresses can be validated.
        return $result;
    } else {
        $result = func_taxjar_Cache('func_taxjar_ValidateAddress', [$address]);
    }

    return $result;
} //}}}

/**
 * Validate VAT number
 * Use cache to get data
 *
 * @param array  $customerInfo Customer info
 *
 * @return array
 */
function func_taxjar_ValidateVat($customerInfo)
{ //{{{
    return func_taxjar_Validate($customerInfo);
} //}}}

/**
 * Get sales tax
 * Shows the sales tax that should be collected for a given order.
 *
 * @param array $products     Products
 * @param array $customerInfo Customer information
 * @param float $shippingCost Shipping cost
 *
 * @return array
 */
function func_taxjar_TaxForOrder($products, $customerInfo, $shippingCost)
{ //{{{
    global $config, $cart;

    $result = false;

    if (
        empty($customerInfo['s_country'])
        && empty($customerInfo['s_zipcode'])
    ) {
        return $result;
    }

    $hasNexus = func_taxjar_HasUsaNexus($customerInfo);
    $nexusStates = func_taxjar_NexusStates();

    if ('US' === $customerInfo['s_country']
        && !empty($nexusStates)
        && !$hasNexus
    ) {
        return $result;
    }

    $orderSubtotal = 0;

    $params = [
        'from_country'  => $config['Company']['location_country'],
        'from_zip'      => $config['Company']['location_zipcode'],
        'from_state'    => $config['Company']['location_state'],
        'from_city'     => $config['Company']['location_city'],
        'from_street'   => $config['Company']['location_address'],
        'to_country'    => $customerInfo['s_country'],
        'to_zip'        => $customerInfo['s_zipcode'],
        'to_state'      => $customerInfo['s_state'],
        'to_city'       => $customerInfo['s_city'],
        'to_street'     => empty($customerInfo['s_address']) ? '' : $customerInfo['s_address'],
        'amount'        => $orderSubtotal,
        'shipping'      => $shippingCost,
        'customer_id'   => empty($customerInfo['email']) ? '' : $customerInfo['email'],
        'line_items'    => [],
    ];

    $params = XC_TaxJar::setExemptionType($params, $customerInfo);

    if ($config['CSI_TaxJar']['taxjar_ship_local'] == $cart['shippingid'] && !empty($cart['shippingid'])) {
        $shippingTo = [
            'to_country'  => $config['Company']['location_country'],
            'to_zip'      => $config['Company']['location_zipcode'],
            'to_state'    => $config['Company']['location_state'],
            'to_city'     => $config['Company']['location_city'],
            'to_street'   => $config['Company']['location_address'],
        ];

        $params = array_merge($params, $shippingTo);
    }

    if ($hasNexus) {
        foreach ($nexusStates as $state) {
            $params['nexus_addresses'][] = [
                'country' => 'US',
                'state'   => $state,
            ];
        }
    }

    if ('US' === $params['to_country']
        && empty($params['to_zip'])
    ) {
        // To Avoid 400 Bad Request – No To Zip, Required When Country Is US
        return $result;
    }

    if ($products) {
        foreach ($products as $key => $product) {
            if ('Y' == $product['free_tax']) {
                continue;
            }

            $orderSubtotal += $product['display_discounted_price'];

            $params['line_items'][$key] = [
                'id'         => XC_TaxJar::getProductUniqId($product),
                'quantity'   => $product['amount'],
                'unit_price' => $product['price'],
                'discount'   => round($product['price'] * $product['amount'] - $product['display_discounted_price'], 10),
            ];

            $params['line_items'][$key] = XC_TaxJar::setProductTaxCode($params['line_items'][$key], $product);
        }

        /*
        if (!empty($orderData['order']['payment_surcharge']) && floatval($orderData['order']['payment_surcharge']) != 0) {#nolint
            $params['line_items'][$key] = [
                'id'                 => 'payment_surcharge',
                'quantity'           => 1,
                'unit_price'         => $orderData['order']['payment_surcharge'],#nolint
                'discount'           => 0,
            ];
        }*/

        $params['amount'] = $orderSubtotal;
    }

    $response = func_taxjar_SendRequest('taxForOrder', $params);

    if (!$response['error']) {
        $data = $response['response'];

        $taxShipping = isset($data['breakdown']) ? $data['breakdown']['shipping']['tax_collectable'] : 0;

        $result['total'] = $data['amount_to_collect'];
        $result['shipping'] = $taxShipping;
        $result['taxjar_response'] = serialize($data);

        if (0 < $data['rate']) {
            $result['taxes'][XC_TAXJAR_TAX_NAME] = [
                'taxid'                     => XC_TAXJAR_TAX_NAME,
                'tax_name'                  => XC_TAXJAR_TAX_NAME,
                'tax_display_name'          => 'Sales Tax',
                'rate_type'                 => '%',
                'rate_value'                => $data['rate'] * 100,
                'price_includes_tax'        => 'N',
                'tax_cost'                  => $data['amount_to_collect'],
                'shipping_taxcost'          => $taxShipping,
                'taxcost_without_shipping'  => $data['amount_to_collect'] - $taxShipping,
            ];
        }
    }

    return $result;
} //}}}

/**
 * List order transactions
 * Lists existing order transactions created through the API.
 *
 * @param mixed $dateRange From-to date or specific date as Unix timestamp
 *
 * @return array
 */
function func_taxjar_ListOrders($dateRange = null)
{ //{{{
    return func_taxjar_SendRequest('listOrders', func_taxjar_DateRange($dateRange));
} //}}}

/**
 * Show/delete order info
 * Shows/Delete an existing order transaction created through the API.
 *
 * @param string  $action Create or update action
 * @param integer $transactionId Transaction ID
 *
 * @return array
 */
function func_taxjar_ShowDeleteOrder($action, $transactionId)
{ //{{{
    $action = strtolower($action);

    if (!in_array($action, ['show', 'delete'])) {
        $result = [
            'error'     => true,
            'response'  => [
                'code'  => 400,
                'msg'   => 'Wrong action submitted. Action must be "show" or "delete".'
            ]
        ];

        func_taxjar_LogEvent($result);
    } else {
        $result = func_taxjar_SendRequest($action . 'Order', $transactionId);
    }

    return $result;
} //}}}

/**
 * Create/Update order transaction
 * Creates/Updates order transaction.
 *
 * @param array $orderData Order information
 *
 * @return array
 */
function func_taxjar_CreateUpdateOrder($orderData)
{ //{{{
    global $config;

    $result = [];

    if ('Y' == $config['CSI_TaxJar']['taxjar_send_info']) {
        $action = 'create';
        $existingOrders = func_taxjar_ListOrders([strtotime('-3 months'), time()]);

        if (!empty($existingOrders['response'])
            && is_array($existingOrders['response'])
            && in_array($orderData['order']['orderid'], $existingOrders['response'])
        ) {
            $action = 'update';
        }

        $params = [
            'transaction_id'   => $orderData['order']['orderid'],
            'transaction_date' => date('c', $orderData['order']['date']),
            'from_country'     => $config['Company']['location_country'],
            'from_zip'         => $config['Company']['location_zipcode'],
            'from_state'       => $config['Company']['location_state'],
            'from_city'        => $config['Company']['location_city'],
            'from_street'      => $config['Company']['location_address'],
            'to_country'       => $orderData['userinfo']['s_country'],
            'to_zip'           => $orderData['userinfo']['s_zipcode'],
            'to_state'         => $orderData['userinfo']['s_state'],
            'to_city'          => $orderData['userinfo']['s_city'],
            'to_street'        => $orderData['userinfo']['s_address'],
            'amount'           => $orderData['order']['total'] - $orderData['order']['tax'],
            'shipping'         => $orderData['order']['shipping_cost'],
            'sales_tax'        => $orderData['order']['tax'],
            'customer_id'      => $orderData['userinfo']['email'],
            'line_items'       => [],
        ];

        $params = XC_TaxJar::setExemptionType($params, $orderData['userinfo']);

        if ($config['CSI_TaxJar']['taxjar_ship_local'] == $orderData['order']['shippingid'] && !empty($orderData['order']['shippingid'])) {
            $shippingTo = [
                'to_country'  => $config['Company']['location_country'],
                'to_zip'      => $config['Company']['location_zipcode'],
                'to_state'    => $config['Company']['location_state'],
                'to_city'     => $config['Company']['location_city'],
                'to_street'   => $config['Company']['location_address'],
            ];

            $params = array_merge($params, $shippingTo);
        }

        $_i = 0;
        if (!empty($orderData['products'])) {
            foreach ($orderData['products'] as $product) {
                $sales_tax = 0;

                if (isset($product['extra_data']['taxes'])) {
                    foreach ($product['extra_data']['taxes'] as $tax) {
                        $sales_tax += $tax['tax_value'];
                    }
                }

                $params['line_items'][$_i] = [
                    'id'                 => XC_TaxJar::getProductUniqId($product),
                    'quantity'           => $product['amount'],
                    'product_identifier' => $product['productcode'],
                    'description'        => $product['product'],
                    'unit_price'         => $product['price'],
                    'discount'           => round($product['price'] * $product['amount'] - $product['display_discounted_price'], 10),
                    'sales_tax'          => $sales_tax,
                ];

                $params['line_items'][$_i] = XC_TaxJar::setProductTaxCode($params['line_items'][$_i], $product);

                $_i++;
            }
        }

        if (!empty($orderData['order']['payment_surcharge']) && floatval($orderData['order']['payment_surcharge']) != 0) {
            $params['line_items'][$_i++] = [
                'id'                 => 'payment_surcharge',
                'quantity'           => 1,
                'product_identifier' => 'payment_surcharge',
                'description'        => func_get_langvar_by_name('lbl_payment_method_surcharge', false, false, true),
                'unit_price'         => $orderData['order']['payment_surcharge'],
                'discount'           => 0,
                'sales_tax'          => 0,
            ];
        }

        if (!empty($orderData['giftcerts'])) {
            foreach ($orderData['giftcerts'] as $k => $_giftcert) {
                $params['line_items'][$_i++] = [
                    'id'                 => 'Giftcert #' . ++$k,
                    'quantity'           => 1,
                    'product_identifier' => $k . '-' . $_giftcert['orderid'],
                    'description'        => 'Giftcert for ' . $_giftcert['recipient'] . "($_giftcert[recipient_email])" ,
                    'unit_price'         => $_giftcert['amount'],
                    'discount'           => 0,
                    'sales_tax'          => 0,
                ];
            }
        }

        $result = func_taxjar_SendRequest($action . 'Order', $params);
    }

    return $result;
} //}}}

/**
 * List refund transactions
 * Lists existing refund transactions created through the API.
 *
 * @param mixed $dateRange From-to date or specific date as Unix timestamp
 *
 * @return array
 */
function func_taxjar_ListRefunds($dateRange = null)
{ //{{{
    return func_taxjar_SendRequest('listRefunds', func_taxjar_DateRange($dateRange));
} //}}}

/**
 * Show/delete refund transaction
 * Shows/Delete an existing refund transaction created through the API.
 *
 * @param string $action Show or delete action
 * @param integer $transactionId Transaction ID
 *
 * @return array
 */
function func_taxjar_ShowDeleteRefund($action, $transactionId)
{ //{{{
    $action = strtolower($action);

    if (!in_array($action, ['show', 'delete'])) {
        $result = [
            'error'     => true,
            'response'  => [
                'code'  => 400,
                'msg'   => 'Wrong action submitted. Action must be "show" or "delete".'
            ]
        ];

        func_taxjar_LogEvent($result);
    } else {
        $result = func_taxjar_SendRequest($action . 'Refund', $transactionId . '-refund');
    }

    return $result;
} //}}}

/**
 * Create/Update Refund transaction
 * Creates/Updates Refund transaction.
 *
 * @param integer $Returnid Return id
 *
 * @return array
 *
 * TO DO - refactor for only the refunded items / amounts
 */
function func_taxjar_CreateUpdateRefund($returnid)
{ //{{{
    global $config;

    $result = [];

    if ('Y' == $config['CSI_TaxJar']['taxjar_send_info']) {
        $action = 'create';
        $existingRefunds = func_taxjar_ListRefunds([strtotime('-6 months'), time()]);
        $_prefix_id = '-refund';

        if (!empty($existingRefunds['response'])
            && is_array($existingRefunds['response'])
            && in_array($returnid . $_prefix_id, $existingRefunds['response'])
        ) {
            $action = 'update';
        }

        $refund = func_return_data($returnid);

        $sales_tax = 0;

        if (isset($refund['product']['extra_data']['taxes'])) {
            foreach ($refund['product']['extra_data']['taxes'] as $tax) {
                $sales_tax += $tax['tax_value'];
            }
        }

        $qty = ('create' == $action || empty($refund['returned_amount'])) ? $refund['amount'] : $refund['returned_amount'];
        $qty_proportion = $qty / max(1, $refund['product']['amount']);// Proportion Of The Return

        $params = [
            'transaction_id'           => $refund['returnid'] . $_prefix_id,
            'transaction_reference_id' => $refund['order']['orderid'],
            'transaction_date'         => date('c', $refund['date']),
            'from_country'             => $config['Company']['location_country'],
            'from_zip'                 => $config['Company']['location_zipcode'],
            'from_state'               => $config['Company']['location_state'],
            'from_city'                => $config['Company']['location_city'],
            'from_street'              => $config['Company']['location_address'],
            'to_country'               => $refund['userinfo']['s_country'],
            'to_zip'                   => $refund['userinfo']['s_zipcode'],
            'to_state'                 => $refund['userinfo']['s_state'],
            'to_city'                  => $refund['userinfo']['s_city'],
            'to_street'                => $refund['userinfo']['s_address'],
            'amount'                   => $refund['product']['display_discounted_price'] * $qty_proportion * -1,
            'shipping'                 => 0,
            'sales_tax'                => $sales_tax * $qty_proportion * -1,
            'customer_id'              => $refund['userinfo']['email'],
            'line_items'               => [
                [
                    'id'                 => XC_TaxJar::getProductUniqId($refund['product']),
                    'quantity'           => $qty,
                    'product_identifier' => $refund['product']['productcode'],
                    'description'        => $refund['product']['product'],
                    'unit_price'         => $refund['product']['price'] * -1,
                    'discount'           => (round($refund['product']['price'] * $refund['product']['amount'] - $refund['product']['display_discounted_price'], 10)) * $qty_proportion * -1,
                    'sales_tax'          => $sales_tax * $qty_proportion * -1,
                ]
            ],
        ];
        $params = XC_TaxJar::setExemptionType($params, $refund['userinfo']);

        $params['line_items'][0] = XC_TaxJar::setProductTaxCode($params['line_items'][0], $refund['product']);

        if ($config['CSI_TaxJar']['taxjar_ship_local'] == $refund['order']['shippingid'] && !empty($refund['order']['shippingid'])) {
            $shippingTo = [
                'to_country'  => $config['Company']['location_country'],
                'to_zip'      => $config['Company']['location_zipcode'],
                'to_state'    => $config['Company']['location_state'],
                'to_city'     => $config['Company']['location_city'],
                'to_street'   => $config['Company']['location_address'],
            ];

            $params = array_merge($params, $shippingTo);
        }

        $result = func_taxjar_SendRequest($action . 'Refund', $params);
    }

    return $result;
} //}}}

/**
 * List customers
 * Lists existing customers created through the API.
 *
 * @return array
 */
function func_taxjar_ListCustomers()
{ //{{{
    return func_taxjar_SendRequest('listCustomers');
} //}}}

/**
 * Show/delete customer
 * Shows or deletes an existing customer created through the API.
 *
 * @param string $action Show or delete action
 * @param string $customerId Customer ID
 *
 * @return array
 */
function func_taxjar_ShowDeleteCustomer($action, $customerId)
{ //{{{
    $action = strtolower($action);

    if (!in_array($action, ['show', 'delete'])) {
        $result = [
            'error'     => true,
            'response'  => [
                'code'  => 400,
                'msg'   => 'Wrong action submitted. Action must be "show" or "delete".'
            ]
        ];

        func_taxjar_LogEvent($result);
    } else {
        $result = func_taxjar_SendRequest($action . 'Customer', $customerId);
    }

    return $result;
} //}}}

/**
 * Create/update customer
 * Creates or updates a customer.
 *
 * @param array $customerInfo Customer information
 *
 * @return array
 */
function func_taxjar_CreateUpdateCustomer($customerInfo)
{ //{{{
    global $config;

    $result = [];

    if ('Y' == $config['CSI_TaxJar']['taxjar_send_info']) {
        $action = 'create';
        $existingCustomers = func_taxjar_ListCustomers();

        if (!empty($existingCustomers['response'])
            && is_array($existingCustomers['response'])
            && in_array($customerInfo['email'], $existingCustomers['response'])
        ) {
            $action = 'update';
        }

        if (isset($customerInfo['company'])
            && !empty($customerInfo['company'])
        ) {
            $name = $customerInfo['company'];
        } else {
            $name = $customerInfo['firstname'] . ' ' . $customerInfo['lastname'];
        }

        $params = [
            'customer_id'       => $customerInfo['email'],
            'exemption_type'    => ('Y' == $customerInfo['tax_exempt']) ? 'other' : 'non_exempt',
            'name'              => $name,
            'country'           => $customerInfo['b_country'],
            'zip'               => $customerInfo['b_zipcode'],
            'state'             => $customerInfo['b_state'],
            'city'              => $customerInfo['b_city'],
            'street'            => $customerInfo['b_address'],
        ];

        if ('US' == $customerInfo['s_country']) {
            $params['exempt_regions'] = [
                [
                    'country' => 'US',
                    'state'   => $customerInfo['s_state'],
                ],
            ];
        }

        $result = func_taxjar_SendRequest($action . 'Customer', $params);
    }

    return $result;
} //}}}

/**
 * Rates for location
 * Show the sales tax rates for a given location
 *
 * @param array  $customerInfo Customer info
 * @param string $type Address type - billing or shipping
 *
 * @return array
 */
function func_taxjar_RatesForLocation($customerInfo, $type = 's')
{ //{{{
    $params = [
        'country' => $customerInfo[$type . '_country'],
        'state'   => $customerInfo[$type . '_state'],
        'zipcode' => $customerInfo[$type . '_zipcode'],
        'city'    => $customerInfo[$type . '_city'],
        'street'  => $customerInfo[$type . '_address'],
    ];

    return empty($customerInfo[$type . '_zipcode']) ? [] : func_taxjar_SendRequest('ratesForLocation', $params);
} //}}}

/**
 * Validate address
 * Validates a customer address and returns back a collection of address matches.
 * Address validation requires a TaxJar Plus subscription.
 *
 * @param array  $address Customer address
 *
 * @return array
 */
function func_taxjar_ValidateAddress($address)
{ //{{{
    $errors = [];

    $client = func_taxjar_Client();

    if ($client) {
        try {
            $config = func_taxjar_Config();

            $address['zip'] = $address['zipcode'] . (empty($address['zip4']) ? '' : "-$address[zip4]");
            $address['street'] = $address['address'];
            $address = array_intersect_key($address, array_flip(array('country' , 'state' , 'zip', 'city', 'street')));
            $result = $client->validateAddress($address);

        } catch (TaxJar\Exception $e) {
            func_taxjar_LogEvent($e);

            $errors[] = [
                'fields'    => [],
                'error'     => 'TaxJar Address Validation exception: ' . $e->getMessage()
            ];
        } catch (\Exception $e) {
            func_taxjar_LogEvent($e);

            $errors[] = [
                'fields'    => [],
                'error'     => 'TaxJar Address Validation exception: ' . $e->getMessage()
            ];
        }

        if ('Y' == $config['taxjar_debug']) {
            $debugInfo = [
                'action'    => 'validateAddress',
                'params'    => $address,
                'result'    => empty($result) ? false : $result,
                'error'     => $errors,
            ];

            x_log_add('taxjar_debugs', $debugInfo, true);
        }
    }

    return [
        'status' => empty($errors),
        'errors' => $errors
    ];
} //}}}

/**
 * Validate VAT number
 * Validates an existing VAT identification number against VIES.
 *
 * @param array  $customerInfo Customer info
 *
 * @return array
 */
function func_taxjar_Validate($customerInfo)
{ //{{{
    $params = [
        'vat' => $customerInfo['tax_number'],
    ];

    return empty($customerInfo['tax_number']) ? [] : func_taxjar_SendRequest('validate', $params);
} //}}}

/**
 * Summarize tax rates for all regions
 * Retrieve minimum and average sales tax rates by region as a backup.
 *
 * @return array
 */
function func_taxjar_SummaryRates()
{ //{{{
    return func_taxjar_SendRequest('summaryRates');
} //}}}
