<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * @category   Modules
 * @package    X-Cart
 * @subpackage Two-Factor authentication via Authy
 * @author     Ildar Amankulov
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    3f64c27f32722bbd2ac6f4428a96f705235bcb26, v4 (xcart_4_7_12), 2020-04-23 15:05:41, func.php, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */

if (!defined('XCART_START')) {
    header('Location: ../../');
    die('Access denied');
}

function func_twofactor_validate_phone($address) { //{{{
    global $config, $current_area;

    if (
        $current_area == 'C'
        && $config['TwoFactorAuth']['twofactor_verify_' . $current_area] == 'N'
    ) {
        return false;
    }

    $is_error = empty($address['phone']);

    if (!$is_error) {
        $is_error = (func_twofactor_get_phone($address) == FALSE);
    }

    if ($is_error) {
        $is_error = array(
            'fields' => array('country','phone'),
            'error'  => func_get_langvar_by_name('err_twofactor_invalid_phone')
        );
    }

    return $is_error;

} //}}}

function func_twofactor_on_phone_update($userid) { //{{{

    $return = FALSE;

    $phone = func_twofactor_get_phone($userid);

    if (!empty($phone)) {

//        func_twofactor_delete_user($userid);

        $return = func_twofactor_register_user($userid, $phone);

    }

    return $return;

} //}}}

function func_twofactor_is_waiting_for_token($area) { //{{{

    global $config, $twofactor_verify;
    x_session_register('twofactor_verify');

    return (
            $config['TwoFactorAuth']['twofactor_verify_' . $area] == 'Y'
            && !empty($twofactor_verify[$area])
            && !defined('DISABLE_TWOFACTOR')
       );
} //}}}

function func_twofactor_on_check_useraccount($area) { //{{{

    global $PHP_SELF;

    $script = basename($PHP_SELF);

    // get_block.php loads as from C even when called from A
    // to avoid conflicts skip check for safe-to-load blocks
    if (
        $script == 'get_block.php'
        && (
            $_GET['block'] == 'provider_commission_data'
            || $_GET['block'] == 'latest_xcart_news'
            || $_GET['block'] == 'admin_menu'
        )
    ) {
        return;
    }

    if (func_twofactor_is_waiting_for_token($area)) {

        global $login, $smarty;
        x_session_register('login');
        $login = '';
        $smarty->assign('login', '');

        if (
            $script != 'twofactor_token.php'
            && ($script != 'login.php' || empty($_GET['mode']) || $_GET['mode'] != 'logout')
        ) {
            if (func_is_ajax_request()) {
                func_reload_parent_window();
            }
            func_header_location('twofactor_token.php');
        }

    }

} //}}}

function func_twofactor_unset_token_waiting($area) { //{{{
    global $twofactor_verify, $active_modules;
    x_session_register('twofactor_verify', array());

    if (!empty($twofactor_verify[$area])) {

        if (!empty($active_modules['Simple_Mode']) && ($area == 'P' || $area == 'A')) {
            unset($twofactor_verify['A']);
            unset($twofactor_verify['P']);
        } else {
            unset($twofactor_verify[$area]);
        }

        if (empty($twofactor_verify)) {
            x_session_unregister('twofactor_verify');
        }
    }

} //}}}

function func_twofactor_on_logout($area) { //{{{

    func_twofactor_unset_token_waiting($area);

    if (func_is_ajax_request()) {
        func_reload_parent_window();
    }

} //}}}

function func_twofactor_on_authenticate($userid, $area, $saved_login = '') { //{{{

    global $config, $active_modules;

    if (
        $config['TwoFactorAuth']['twofactor_verify_' . $area] != 'Y'
        || defined('DISABLE_TWOFACTOR')
    ) {
        return FALSE;
    }

    // Check phone always to make sure it was not broken when module was disabled
    $phone = func_twofactor_get_phone($userid);

    if (!empty($phone)) {
        $authyid = func_twofactor_get_authyid($userid);

        if (empty($authyid)) {
            // Automatically register user in Authy if not yet registered
            $authyid = func_twofactor_register_user($userid, $phone);
        }
    }

    if (!empty($authyid)) {

        global $twofactor_verify;
        x_session_register('twofactor_verify', array());

        $sms_result = func_twofactor_send_sms($authyid);

        if (
            is_object($sms_result) && $sms_result->success && $sms_result->cellphone
            && !empty($phone['parsed_phone'])
            && substr($phone['parsed_phone'], -2) != substr($sms_result->cellphone, -2)
        ) {
            $new_authyid = func_twofactor_register_user($userid, $phone);
            if ($new_authyid != $authyid) {
                $sms_result = func_twofactor_send_sms($new_authyid);
            }
        }

        $twofactor_verify[$area] = array(
            'saved_login' => $saved_login,
            'sms_sent' => is_object($sms_result) ? $sms_result->success : $sms_result,
            'redirect_url' => 'home.php',
        );

        if (!empty($active_modules['Simple_Mode']) && ($area == 'P' || $area == 'A')) {
            $twofactor_verify['A'] = $twofactor_verify['P'] = $twofactor_verify[$area];
        }

    } else {

        return FALSE;

    }

} //}}}

function func_twofactor_on_login($userid, $area, $redirect_url) { //{{{

    global $top_message, $config, $active_modules, $twofactor_verify;

    $fail_result = array(FALSE, $redirect_url);

    if (
        $config['TwoFactorAuth']['twofactor_verify_' . $area] != 'Y'
        || defined('DISABLE_TWOFACTOR')
    ) {
        return $fail_result;
    }

    $phone = func_twofactor_get_phone($userid);

    if (empty($phone)) {
        if (func_is_ajax_request()) {

            // Set popup to reload page on close and add OK button
            $lbl_ok = func_get_langvar_by_name('lbl_ok', NULL, FALSE, TRUE);
            $jscode = <<<JS
var buttons = {};
buttons['$lbl_ok'] = function() {
    o.close();
}

$(o.element).dialog(
    {
        close: function() {
            window.location='$redirect_url';
        },
        buttons: buttons
});
JS;
            func_register_ajax_message(
                'popupDialogCall',
                array(
                    'action' => 'jsCall',
                    'toEval' => $jscode
                )
            );

            // Show no phone warning
            func_register_ajax_message(
                'popupDialogCall',
                array(
                    'action' => 'load',
                    'src'    => 'twofactor_token.php?show_message=no_phone'
                )
            );
            return array(TRUE, $redirect_url);

        } else {
            $top_message = array(
                'content' => func_get_langvar_by_name('msg_twofactor_no_phone_login', null, false, true),
                'type' => 'E'
            );

            return $fail_result;
        }
    }

    x_session_register('twofactor_verify');

    if (!empty($twofactor_verify[$area])) {

        $twofactor_verify[$area]['redirect_url'] = $redirect_url;

        if (!empty($active_modules['Simple_Mode']) && ($area == 'P' || $area == 'A')) {
            $twofactor_verify['A'] = $twofactor_verify['P'] = $twofactor_verify[$area];
        }

        if (func_is_ajax_request()) {
            // Set popup to logout on close
            func_register_ajax_message(
                'popupDialogCall',
                array(
                    'action' => 'jsCall',
                    'toEval' => '$(o.element).dialog({close: function() {window.location="login.php?mode=logout";}});'
                )
            );
            // Open token verify page
            func_register_ajax_message(
                'popupDialogCall',
                array(
                    'action' => 'load',
                    'src'    => 'twofactor_token.php'
                )
            );
        }

        return array(true, 'twofactor_token.php');

    } else {
        $top_message = array(
            'content' => func_get_langvar_by_name('msg_twofactor_authy_error'),
            'type' => 'E'
        );
        return $fail_result;
    }

} //}}}

function func_twofactor_verify_token($token, $userid) { //{{{

    $return = FALSE;

    $authyid = func_twofactor_get_authyid($userid);

    if ($authyid) {
        $response = func_twofactor_authy_verify_token($token, $authyid, TRUE);
        $return = $response->success;
    }

    return $return;

} //}}}

function func_twofactor_get_phone($userid) { //{{{

    x_load('user');

    $phone = FALSE;

    if (!is_array($userid)) {
        // userid is passed
        $address_book = func_get_address_book($userid);
        if (!empty($address_book)) {
            // Select phone from addressbook, billing is higher priority than shipping
            foreach ($address_book as $address) {
                if (!empty($address['phone'])) {
                    if ((empty($from_address) && $address['default_s']) || $address['default_b']) { #nolint
                        $from_address = $address;
                    }
                }
            }
        }
    } elseif (!empty($userid['phone'])) {
        // address array is passed
        $from_address = $userid;
    }

    if (!empty($from_address)) {
        $phone = array('phone' => $from_address['phone'], 'country' => $from_address['country']);
    }

    if (!empty($phone)) {
        $country_code = func_twofactor_detect_phone_country($phone['phone'], $phone['country']);
        if (!empty($country_code)) {
            $phone['country_code'] = $country_code;
            $phone['parsed_phone'] = preg_replace('/[^0-9]/','', $phone['phone']);

            if (strpos($phone['parsed_phone'], $country_code) === 0) {
                $phone['parsed_phone'] = substr($phone['parsed_phone'], strlen($country_code));
            } else {
                $phone = FALSE;
            }

        } else {
            $phone = FALSE;
        }
    }

    return $phone;

} //}}}

function func_twofactor_detect_phone_country($phone, $country) { //{{{

    global $sql_tbl;

    $country_code = '';

    if (preg_match('/^\+([0-9]{1,3})[\s()\-]/', $phone, $country_code_match)) {
        // Get country dialing code from phone
        $country_code = $country_code_match[1];
    } else if (strpos($phone, '1-') === 0) {
        // US country code
        $country_code = '1';
    } else if (!empty($country)) {
        // Try to match dialing code using database
        $country_code = func_query_first_cell("SELECT code FROM $sql_tbl[twofactor_dialing_codes] WHERE country='$country'");
    }

    return $country_code;

} //}}}

function func_twofactor_register_user($userid, $phone = FALSE) { //{{{

    global $sql_tbl;

    if (empty($phone)) {
        $phone = func_twofactor_get_phone($userid);
    }

    if (!empty($phone)) {

        $userinfo = func_userinfo($userid);

        $_md5_hash = md5(serialize(array($userinfo['email'], $phone['parsed_phone'], $phone['country_code'])));

        $cached_data = func_query_first("SELECT * FROM $sql_tbl[twofactor_users] WHERE userid='$userid' AND data_hash='$_md5_hash'");
        if (empty($cached_data)) {
            $response = func_twofactor_authy_register_user($userinfo['email'], $phone['parsed_phone'], $phone['country_code']);

            if ($response->success && isset($response->user)) {
                $authyid = intval($response->user->id);
                if (!empty($authyid)) {
                    db_query("REPLACE INTO $sql_tbl[twofactor_users] (userid, authyid, data_hash) VALUES ('$userid', '$authyid', '$_md5_hash')");
                    return $authyid;
                }
            }
        } else {
            return $cached_data['authyid'];
        }
    }

    return FALSE;

} //}}}

function func_twofactor_delete_user($userid) { //{{{

    global $sql_tbl;

    $return = FALSE;

    $authyid = func_twofactor_get_authyid($userid);

    if ($authyid) {
        $response = func_twofactor_authy_delete_user($authyid);
        $return = $response->success;
        if ($return) {
            db_query("DELETE FROM $sql_tbl[twofactor_users] WHERE userid='$userid'");
        }
    }

    return $return;

} //}}}

function func_twofactor_send_sms($_authyid) { //{{{

    global $config;

    $return = FALSE;

    if ($_authyid) {
        $response = func_twofactor_authy_send_sms($_authyid, ($config['TwoFactorAuth']['twofactor_force_sms'] == 'Y'));
        $return = $response;
        if ($return && isset($response->message)) {
            if (stripos($response->message, 'Ignored') !== FALSE) {
                $return = FALSE;
            }
        }
    }

    return $return;

} //}}}

function func_twofactor_get_app_details() { //{{{

    $return = FALSE;

    $data = func_twofactor_authy_application_details();

    if ($data->success && isset($data->app)) {
        $return = (array)$data->app;
    }

    return $return;

} //}}}

function func_twofactor_get_authyid($userid) { //{{{

    global $sql_tbl;

    return func_query_first_cell("SELECT authyid FROM $sql_tbl[twofactor_users] WHERE userid='$userid'");

} //}}}

function func_twofactor_redefine_config(&$configuration, $key) { //{{{

    global $active_modules;

    if (!empty($active_modules['Simple_Mode']) && $configuration[$key]['name'] == 'twofactor_verify_P') {
        unset($configuration[$key]);
    }

} //}}}

function func_twofactor_on_config_update() { //{{{

    global $config;

    // Update config vars
    foreach ($config['TwoFactorAuth'] as $option => $value) {
        if (array_key_exists($option, $_POST)) {
            $config['TwoFactorAuth'][$option] = (($_POST[$option] == 'on') ? 'Y' : $_POST[$option] ) ;
        } elseif ($option != 'twofactor_test_mode') {
            $config['TwoFactorAuth'][$option] = '';
        }
    }

    func_twofactor_define_profile_fields();

    func_twofactor_get_app_details();

} //}}}

function func_twofactor_define_profile_fields($get_from_post = FALSE) { //{{{

    global $sql_tbl, $config, $active_modules;

    $usertypes_to_enable = array();
    foreach (array('A', 'P', 'C') as $usertype) {

        if (
            $get_from_post && !empty($_POST['twofactor_verify_' . $usertype])
            || !$get_from_post && $config['TwoFactorAuth']['twofactor_verify_' . $usertype] == 'Y'
        ) {
            $usertypes_to_enable[] = $usertype;
            if ($usertype == 'C') {
                $usertypes_to_enable[] = 'H';
            } else if (!empty($active_modules['Simple_Mode']) && $usertype == 'A') {
                $usertypes_to_enable[] = 'P';
            }
        }

    }

    $current_values = func_query_first_cell("SELECT value FROM $sql_tbl[config] WHERE name='address_book_fields' AND category='User_Profiles'");
    $current_values = unserialize($current_values);

    if (empty($current_values)) {
        global $default_address_book_fields;
        $current_values = array();
        foreach($default_address_book_fields as $k => $v) {
            foreach (array('avail', 'required') as $key) {
                if ($v[$key] == 'Y') {
                    $$key ='APBCH';
                } else {
                    $$key = '';
                    foreach ($v[$key] as $ak => $av) {
                        if ($av == 'Y') {
                            $$key .= $ak;
                        }
                    }
                }
            }
            $current_values[] = array(
                'title'     => func_get_default_field($k),
                'field'     => $k,
                'avail'     => $avail, #nolint
                'required'  => $required #nolint
            );
        }
    }

    $changed = FALSE;

    foreach ($current_values as $k => $v) {
        if ($v['field'] == 'phone' || $v['field'] == 'country') {
            foreach ($usertypes_to_enable as $usertype) {
                foreach (array('avail', 'required') as $key) {
                    if (strpos($v[$key], $usertype) === FALSE) {
                        $changed = TRUE;
                        $current_values[$k][$key] .= $usertype;
                    }
                }
            }
        }
    }

    if ($changed) {
        $tmp_string = addslashes(serialize($current_values));
        db_query("REPLACE INTO $sql_tbl[config] (name, value, category, defvalue, variants) VALUES ('address_book_fields', '$tmp_string', 'User_Profiles', '', '')");
        func_data_cache_clear('get_default_fields');

        x_log_flag('log_activity', 'ACTIVITY', "TwoFactorAuth module has changed 'User Profiles::register_fields' option to '$tmp_string'");
    }

} //}}}

function func_twofactor_on_module_toggle($module_name, $active, $prev_result = '') { //{{{
    if ($module_name == 'TwoFactorAuth' && $active) {
        func_twofactor_define_profile_fields();
    }

    return $prev_result;
} //}}}

function func_twofactor_init() { //{{{

    func_add_event_listener('module.ajax.toggle', 'func_twofactor_on_module_toggle');

} //}}}

// ------------------ Authy Api --------------------

function func_twofactor_authy_request($request, $params = array(), $method = 'GET') { //{{{

    global $config;

    x_load('http');

    // https://www.twilio.com/docs/authy/api
    $host = (!$config['TwoFactorAuth']['twofactor_test_mode']) ? 'api.authy.com' : 'sandbox-api.authy.com';
    $path =  '/protected/json/' . $request;

    if (!is_array($params)) {
        $params = array();
    }
    $params['api_key'] = trim($config['TwoFactorAuth']['twofactor_api_key']);

    if (!$config['TwoFactorAuth']['twofactor_test_mode']) {
        list($a, $return) = func_twofactor_authy_https_request($method, $host, $path, $params);
    } else {
        list($a, $return) = func_twofactor_authy_http_request($method, $host, $path, $params);
    }

    $status_code = 0;
    if (!empty($a)) {
        if (preg_match('/HTTP\/.*\s*([0-9][0-9][0-9])\s*/i', $a, $tmp)) {
            $status_code = $tmp[1];
        }
    }

    if (!empty($return) && ($result = func_json_decode($return))) {
        $result->statusCode = $status_code;
        $result->success = ($result->success == 'true');
    } else {
        $result = new stdClass();
        $result->statusCode = $status_code;
        $result->success = FALSE;
    }

    if (defined('TWOFACTOR_DEBUG')) {$params['api_key'] = substr_replace($params['api_key'], '*', 3);
        x_log_add('authy', "Request:\n" . $host . $path . "\n" . print_r($params,TRUE). "\nResponse:\n" . print_r($result,TRUE) . "\nRow Response:\n" . print_r($return, TRUE));
    }

    return $result;

} //}}}

function func_twofactor_authy_http_request($method, $host, $path, $params = array()) { //{{{

    $post = array();
    if (!empty($params)) {
        foreach ($params as $key => $value) {
            $post[] = $key . '=' . $value;
        }
    }

    if ($method == 'GET') {
        $result = func_http_get_request($host, $path, implode('&', $post));
    } else {
        $result = func_http_post_request($host, $path, implode('&', $post));
    }

    return $result;

} //}}}

function func_twofactor_authy_https_request($method, $host, $path, $params = array()) { //{{{

    $post = '';
    if (!empty($params)) {
        $post .= '?';
        foreach ($params as $key => $value) {
            $post .= '&' . urlencode($key) . '=' . urlencode($value);
        }
    }

    $url = 'https://' . $host . $path;
    if ($method == 'GET') {
        $result = func_https_request('GET', $url . $post);
    } else {
        $result = func_https_request('POST', $url, $post);
    }

    return $result;

} //}}}

function func_twofactor_authy_verify_token($token, $authyId, $forceVerification) { //{{{

    $params = array();
    if ($forceVerification) {
        $params['force'] = 'true';
    }

    return func_twofactor_authy_request('verify/' . $token . '/' . $authyId, $params);

} //}}}

function func_twofactor_authy_register_user($email, $cellphone, $countryCode = 0) { //{{{

    $params = array(
        'user[email]' => $email,
        'user[cellphone]' => $cellphone,
    );
    if (!empty($countryCode)) {
        $params['user[country_code]'] = $countryCode;
    }

    return func_twofactor_authy_request('users/new', $params, 'POST');

} //}}}

function func_twofactor_authy_delete_user($authyId) { //{{{

    return func_twofactor_authy_request('users/delete/' . $authyId, array(), 'POST');

} //}}}

function func_twofactor_authy_send_sms($authyId, $forceSend = FALSE) { //{{{

    $params = array();
    if ($forceSend) {
        $params['force'] = 'true';
    }

    return func_twofactor_authy_request('sms/' . $authyId, $params);

} //}}}

function func_twofactor_authy_application_details() { //{{{

    $params = array(
        'referrer' => 'X-Cart'
    );

    return func_twofactor_authy_request('app/details', $params);

} //}}}
