<?php
/* vim: set ts=4 sw=4 sts=4 et: */
/*****************************************************************************\
+-----------------------------------------------------------------------------+
| X-Cart Software license agreement                                           |
| Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>         |
| All rights reserved.                                                        |
+-----------------------------------------------------------------------------+
| PLEASE READ  THE FULL TEXT OF SOFTWARE LICENSE AGREEMENT IN THE "COPYRIGHT" |
| FILE PROVIDED WITH THIS DISTRIBUTION. THE AGREEMENT TEXT IS ALSO AVAILABLE  |
| AT THE FOLLOWING URL: https://www.x-cart.com/license-agreement-classic.html |
|                                                                             |
| THIS AGREEMENT EXPRESSES THE TERMS AND CONDITIONS ON WHICH YOU MAY USE THIS |
| SOFTWARE PROGRAM AND ASSOCIATED DOCUMENTATION THAT QUALITEAM SOFTWARE LTD   |
| (hereinafter referred to as "THE AUTHOR") OF REPUBLIC OF CYPRUS IS          |
| FURNISHING OR MAKING AVAILABLE TO YOU WITH THIS AGREEMENT (COLLECTIVELY,    |
| THE "SOFTWARE"). PLEASE REVIEW THE FOLLOWING TERMS AND CONDITIONS OF THIS   |
| LICENSE AGREEMENT CAREFULLY BEFORE INSTALLING OR USING THE SOFTWARE. BY     |
| INSTALLING, COPYING OR OTHERWISE USING THE SOFTWARE, YOU AND YOUR COMPANY   |
| (COLLECTIVELY, "YOU") ARE ACCEPTING AND AGREEING TO THE TERMS OF THIS       |
| LICENSE AGREEMENT. IF YOU ARE NOT WILLING TO BE BOUND BY THIS AGREEMENT, DO |
| NOT INSTALL OR USE THE SOFTWARE. VARIOUS COPYRIGHTS AND OTHER INTELLECTUAL  |
| PROPERTY RIGHTS PROTECT THE SOFTWARE. THIS AGREEMENT IS A LICENSE AGREEMENT |
| THAT GIVES YOU LIMITED RIGHTS TO USE THE SOFTWARE AND NOT AN AGREEMENT FOR  |
| SALE OR FOR TRANSFER OF TITLE. THE AUTHOR RETAINS ALL RIGHTS NOT EXPRESSLY  |
| GRANTED BY THIS AGREEMENT.                                                  |
+-----------------------------------------------------------------------------+
\*****************************************************************************/

/**
 * The Free Currency Converter API
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage Modules
 * @author     Ruslan R. Fazlyev <rrf@x-cart.com>
 * @copyright  Copyright (c) 2001-present Qualiteam software Ltd <info@x-cart.com>
 * @license    https://www.x-cart.com/license-agreement-classic.html X-Cart license agreement
 * @version    603639bde8626c957b10f92eb26b0ec34734b7af, v4 (xcart_4_7_12), 2020-03-10 19:38:58, mod_currencyconverterapi.php, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */
namespace XCart\Modules\MultiCurrency\currencyconverterapi;

if ( !defined('XCART_START') ) { header("Location: ../../"); die("Access denied"); }

x_load('http');

/**
 * OnlineCurrencyRates  class
 */
class OnlineCurrencyRates
{
    /**
     * URL to post request for rate
     *
     * @var string
     */
    protected $free_url = 'http://free.currconv.com/api/v7/convert';
    protected $paid_url = 'http://api.currconv.com/api/v7/convert';
    protected $pre_paid_url = 'http://prepaid.currconv.com/api/v7/convert';
    protected $is_critical_error = false;

    /**
     * Error message
     *
     * @var string
     */
    private $error = NULL;


    /**
     * Return error
     *
     * @return string
     */
    public function getError()
    {//{{{
        return $this->error;
    }//}}}


    public function isCriticalError()
    {//{{{
        return $this->is_critical_error;
    }//}}}

    /**
     * Get currency conversion rate
     *
     * @param string $from Source currency code (alpha-3)
     * @param string $to   Destination currency code (alpha-3)
     *
     * @return float
     */
    public function getRate($from, $to, $params = array())
    {//{{{
        $result = NULL;

        if (empty($params['mc_currencyconverterapi_account'])) {
            $url = parse_url($this->free_url);
        } elseif ($params['mc_currencyconverterapi_account'] == 'prepaid') {
            $url = parse_url($this->pre_paid_url);
        } elseif ($params['mc_currencyconverterapi_account'] == 'paid') {
            $url = parse_url($this->paid_url);
        }

        $data = array(
            'compact'    => 'ultra',
            'q' => $from . '_' . $to,
        );

        if (!empty($params['mc_currencyconverterapi_apikey'])) {
            $data['apiKey'] = $params['mc_currencyconverterapi_apikey'];
        }

        $postData = array();

        foreach ($data as $k => $v) {
            $postData[] = "$k=$v";
        }

        list($header, $response) = func_http_get_request($url['host'], $url['path'], implode('&', $postData));

        if ($response) {
            $rate = $this->parseResponse($from, $to, $response);

            if ($rate) {
                $result = doubleval($rate);
            }
        } else {
            $this->is_critical_error = true;
        }

        return $result;
    }//}}}


    /**
     * Parse server response
     *
     * @param string $from     Source currency code (alpha-3)
     * @param string $to       Destination currency code (alpha-3)
     * @param string $response Server response
     *
     * @return string
     */
    private function parseResponse($from, $to, $response)
    {//{{{
        $json = json_decode($response, true);

        if (!empty($json[$from . '_' . $to])) {
            return $json[$from . '_' . $to];
        } else {
            if (
                !empty($json['status'])
                && $json['status'] == 400
            ) {
                $this->is_critical_error = true;
            }

            if (
                empty($json)
                || strpos($json, '{') === false
            ) {
                $this->is_critical_error = true;
            }

            if (!empty($json['error'])) {
                $this->error = $json['error'];
            }

            return null;
        }
    }//}}}
}
