/* vim: set ts=2 sw=2 sts=2 et: */
/**
 * X-Cart context help widget controller
 * 
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage JS Library
 * @author     Ruslan R. Fazlyev <rrf@x-cart.com> 
 * @version    2a321c65d1ad1429f47434729de546ac23ecdfac, v10 (xcart_4_7_11), 2019-04-25 08:59:19, context_help.js, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */

;(function () {

	var pluginName = 'xcartHelpWidget';

	(function ($, window, document, undefined) {
		var Widget = function (root, opts) {
			var w = this;

			w.results = $('.context-help-results', root);
			w.form    = $('.context-help-form', root),
			w.input   = $('.context-help-input', root);

			w.cache		= {};

			opts = $.extend({}, {
				searchAsYouTypeDelay: 300,
				requestTimeout: 			6000 /* https://bt.x-cart.com/view.php?id=50252 */
			}, opts);

			Handlebars.registerHelper('seeMoreResults', function(results) {
				if (results && results.length > opts.numExpandedResults) {
					var text = opts.seeMoreCaption.replace(/\{\{n\}\}/gi, results.length - opts.numExpandedResults);

					return new Handlebars.SafeString('<li class="text see-more-results"><a href="#">' + text + '</a></li>');
				}

				return '';
			});

			Handlebars.registerHelper('resultVisibility', function(index) {
			  var style = index < opts.numExpandedResults ? '' : 'style="display: none;"';

			  return new Handlebars.SafeString(style);
			});

			var resultsSource = 
				  '{{#if autocorrection}}'
			  + '  <li class="autocorrection text">' + opts.autocorrectionTpl + '</li>'
			  + '{{/if}}'
			  + '{{#each results}}'
			  + '  <li class="page" {{resultVisibility @index}}>'
			  + '    <div class="title">'
			  + '      <a href="{{this.url}}" target="_blank" title="{{{this.title}}}">{{{this.titleHi}}}</a>'
			  + '    </div>'
			  + '    <div class="content">'
			  + '      {{{this.content}}}'
			  + '    </div>'
			  + '  </li>'
			  + '{{/each}}'
			  + '{{seeMoreResults results}}'
			  + '{{#unless results}}'
			  + '  <li class="no-results text">' + opts.noResultsText + '</li>'
			  + '{{/unless}}';

			var errorsSource = 
				  '{{#if connError}}'
			  + '  <li class="connection-error text">' + opts.connErrorText + '</li>'
			  + '{{/if}}';

			var resultsTemplate = Handlebars.compile(resultsSource),
				errorsTemplate = Handlebars.compile(errorsSource);

			function renderSuccess(data) {
				w.results.html(resultsTemplate(data));

				w.results.find('.see-more-results a').on('click',function () { /*CLiCk_Event_deprecated_in_3_3*/
					w.results.find('.page').show();

					$(this).hide();

					return false;
				});
			}

			function renderError(data) {
				w.results.html(errorsTemplate(data));
			}

			function search() {
				var q = w.input.val();

				if (w.cache[q]) {
					renderSuccess(w.cache[q]);

				} else {
					w.results.addClass('searching').block();

					$.ajax(opts.searchApiUrl, {
						type: 'GET',
						dataType: 'jsonp',
						q: q,
						data: {q: q},
						timeout: opts.requestTimeout,
						success: function (data) {
							if (this.q == q) {
								renderSuccess(data);

								w.results.removeClass('searching');

								w.cache[q] = data;
							}
						},
						error: function () {
							if (this.q == q) {
								renderError({connError: true});

								w.results.removeClass('searching');
							}
						}
					});
				}

				return false;
			}

			$('.context-help-form').on('submit',search); /*SuBmit_eVEnT_dePrEcated_in_3_3*/

      if (opts.searchAsYouType) {
        w.input.on('keydown',function () { /*KeDoWn_eVEnT_dePrEcated_in_3_3*/
        	if (w.timeoutId)
        		clearTimeout(w.timeoutId);

          w.timeoutId = setTimeout(function () { w.form.submit(); }, opts.searchAsYouTypeDelay);
        });
      }

			w.autoSearchPerformed = false;
		};

		Widget.prototype = {
			submit: function () {
			},

			autoSearch: function () {
				if (!this.autoSearchPerformed) {
          if (this.input.val())
            this.form.trigger('submit'); /*SuBmit_trigger_deprecated_in_3_3*/

					this.autoSearchPerformed = true;
				}
			}
		};

		$.fn[pluginName] = function (data) {
			var args = arguments;

			return this.each(function () {
				var plugin = $.data(this, 'plugin_' + pluginName);

				if (!plugin) {
					$.data(this, 'plugin_' + pluginName, new Widget(this, data));
				} else {
					if (typeof plugin[data] == 'function') {
						plugin[data].apply(plugin, Array.prototype.slice.call(args, 1));
					}
				}
			});
		};

	})(jQuery, window, document);


	$(function () {

		var dtools = $('.dialog-tools'),
			opts = contextHelpSettings;

		if (dtools.length) {
			var helpHeader = $('<li>', {
				'class': 'dialog-header-help dialog-tools-nonactive',
				'text' : opts.widgetTitle
			});

      helpHeader.on('click', function () { /*CLiCk_Event_deprecated_in_3_3*/
					dialog_tools_activate('help');
					helpWidget.find('.context-help-root')[pluginName]('autoSearch');
				}
      );

			var helpWidget = $('<div>', {
				'class': 'dialog-tools-content dialog-tools-help hidden',
				'text' : 'Help widget root'
			});

			var q = $('h1').text() || $('#location span:last').text();

			helpWidget.html(
					'<div class="context-help-root">'
			  + '  <form method="get" action="#" class="context-help-form">'
			  + '    <input type="text" class="context-help-input" value="' + q + '" placeholder="' + opts.inputPlaceholder + '" />'
			  + '    <input type="submit" class="context-help-submit" value="' + opts.buttonTitle + '" />'
			  + '  </form>'

			  + '  <ul class="context-help-results">'
			  + '  </ul>'

			  + '  <div class="context-help-hide">'
			  + '    <a href="#">' + opts.hideTabCaption + '</a>'
			  + '  </div>'
			  + '  <div class="clearing"></div>'
			  + '</div>'
			);
			
			dtools.find('.dialog-tools-header').append(helpHeader);

			dtools.find('.dialog-tools-box').append(helpWidget);

			dtools.find('.context-help-submit').button();

			helpWidget.find('.context-help-root')[pluginName](contextHelpSettings);

			if ($('.dialog-tools-header li').length == 1) {
				helpHeader.trigger('click'); /*ClickAs_trigger_deprecated_in_3_3*/
			}

			$('.context-help-hide a').on('click',function () { /*CLiCk_Event_deprecated_in_3_3*/
				xAlert(opts.hideHelpTabText, opts.hideTabCaption, 'I');

				return false;
			});
		}

	});

})();
