/* vim: set ts=2 sw=2 sts=2 et: */
/**
 * Functions for Amazon Payments Advanced module
 *
 * @category   X-Cart
 * @package    X-Cart
 * @subpackage JS Library
 * @author     Ildar Amankulov
 * @version    3e0b0d3ab2c4216ed3ed3a86b46dd2eb25ec9f94, v24 (xcart_4_7_12), 2019-06-20 17:25:57, func.js, aim
 * @link       https://www.x-cart.com/
 * @see        ____file_see____
 */

/* global amazon, ajax, store_language, need_shipping, shippingid, OffAmazonPayments, AMAZON_PA_CONST, amazon_pa_buttons_list, lbl_error, msg_being_placed, txt_accept_terms_err, txt_gdpr_accept_terms_err, txt_ajax_error_note */
/* global amazon_pa_place_order_enabled, amazon_pa_address_selected, amazon_pa_payment_selected */

/**
 * Checks address and update UI by provided amazon order reference id
 *
 * @returns {void}
 */
function func_amazon_pa_check_address() {

  $('#opc_shipping').block();
  $('#opc_totals').block();

  $.post('amazon_checkout.php', {
      'mode': 'check_address'
    },
    function (data) {

      if (data === 'error') {
        xAlert(txt_ajax_error_note, lbl_error, 'E');
        return;
      }

      ajax.core.loadBlock($('#opc_shipping'), 'amazon_pa_shipping', {}, function () {

        amazon_pa_address_selected = true;
        func_amazon_pa_on_change_shipping();

      });

      ajax.core.loadBlock($('#opc_totals'), 'amazon_pa_totals');
    }
  );
}

/**
 * Checks payment and update UI by provided amazon order reference id
 *
 * @returns {void}
 */
function func_amazon_pa_check_payment() {

  amazon_pa_payment_selected = true;
  func_amazon_pa_check_checkout_button();
}

/**
 * Checks checkout button availability
 *
 * @returns {void}
 */
function func_amazon_pa_check_checkout_button() {

  if (
    amazon_pa_payment_selected && amazon_pa_address_selected && func_amazon_ps_is_checkout_ready()
  ) {
    // enable place order button
    $('button.place-order-button').removeClass('inactive');
    amazon_pa_place_order_enabled = true;
  } else {
    // disable place order button
    $('button.place-order-button').addClass('inactive');
    amazon_pa_place_order_enabled = false;
  }
}

/**
 * Return TRUE if checkout is ready
 *
 * @returns {Boolean}
 */
function func_amazon_ps_is_checkout_ready() {

  if (
    need_shipping && (undefined === shippingid || shippingid <= 0)
  ) {
    return false;
  }
  return true;
}

/**
 * Updates UI when customer changes shipping method
 *
 * @returns {void}
 */
function func_amazon_pa_on_change_shipping() {

  $('#opc_totals').block();

  shippingid = $('#opc_shipping').find("input[type='radio']:checked").val();

  if (shippingid) {

    $.post(
      'amazon_checkout.php',
      {
        'mode': 'change_shipping',
        'shippingid': shippingid
      },
      function () {
        ajax.core.loadBlock($('#opc_totals'), 'amazon_pa_totals');
      }
    );

  } else {

    $('#opc_totals').unblock();
  }

  func_amazon_pa_check_checkout_button();
}

/**
 * Place order
 *
 * @returns {Boolean}
 */
function func_amazon_pa_place_order(event)
{//{{{
  event.preventDefault();

  if (!amazon_pa_place_order_enabled) {
    return false;
  }

  var checkoboxes2check = [{ selector:'#accept_terms',          msg:txt_accept_terms_err }];

  if (typeof txt_gdpr_accept_terms_err !== 'undefined') {//if $active_modules.EU_Cookie_Law EU_GDPR
    checkoboxes2check.push({ selector:'#accept_gdpr_checkout',  msg: txt_gdpr_accept_terms_err });
  }

  var has_shake_effect = $.effects && $.effects.effect && $.effects.effect.shake;
  for (var index = 0; index < checkoboxes2check.length; ++index) {
    var termsObj = $(checkoboxes2check[index].selector);
    if (termsObj.length && !termsObj[0].checked) {
      if (has_shake_effect) {
        $('button.place-order-button').effect('shake',{'distance':4, times:2});// if $active_modules.EU_Cookie_Law EU_GDPR
      } else {
        xAlert(checkoboxes2check[index].msg, '', 'W');
      }
      termsObj.css('outline','red solid thin');
      termsObj.one('click', function () {$(this).css('outline', '');});
      return false;
    }
  } // for (var index = 0; index < checkoboxes2check.length; ++index)

  // prevent double submission
  amazon_pa_place_order_enabled = false;
  showXCblockUI(msg_being_placed);
  
  if (AMAZON_PA_CONST.allow_sca_workflow) {
    //  PSD2/SCA changes applied only when merchant region is EU/UK
    OffAmazonPayments.initConfirmationFlow(AMAZON_PA_CONST.SID, AMAZON_PA_CONST.OREFID, function(confirmationFlow) {
      func_amazon_ajax_submit_order_form(confirmationFlow);
    });
  } else {
      func_amazon_ajax_submit_order_form(
        {
          is_not_sca : true,
          error:function(){},
          success:function(){}
        }
      );// confirmationFlow mock
  }

  return true;
}/*}}}funC_amAZon_pa_plaCe_order*/

function func_amazon_ajax_submit_order_form(_confirmationFlow) {//{{{
    // listen EVents from sUccEss_url
    $(ajax.messages).on('xcPlaceOrderEvent', function (e, _data) {
      return onPlaceOrderXC(_data, _confirmationFlow);
    });

    var form = $('#checkout_form');
    ajax.query.add(
      {
        type: 'POST',
        url: form.attr('action'),
        data: form.serialize(),
        error: function(a, b, c, d) {
          _confirmationFlow.error(); // abort Amazon Pay InitConfirmationFlow.Mock method is used for US here
          xAlert(txt_ajax_error_note, lbl_error, 'E');
          return false;
        }
      }
    );
}//fuNc_Amazon_ajAx_subMit_ordEr_fORm}}}

function onPlaceOrderXC(in_data, _confirmationFlow)
{//{{{

  if (in_data && in_data.status) {
    _confirmationFlow.success(); // continue Amazon Pay InitConfirmationFlow
    if (_confirmationFlow.is_not_sca && in_data.url) {
      self.location = in_data.url;
    }

    return true;
  }

  _confirmationFlow.error(); // abort Amazon Pay InitConfirmationFlow

  if (in_data.url) {
    self.location = in_data.url;
  }

  return false;
}/*}}}oNPLaceoRderXC*/

function func_amazon_pa_get_checkout_widget_design()
{
  var design = {
    size: {width: '400px', height: '260px'}
  };

  if (isXCResponsiveSkinOpenOnMobileDevice()) {
    delete design['size'];
    if ($(window).width() < 568) {
      design['designMode'] = 'smartphoneCollapsible';
    } else {
      design['designMode'] = 'responsive';
    }
  }

  return design;
}

/**
 * Initialize buttons widgets
 *
 * @returns {void}
 */
function func_amazon_pa_init_buttons()
{
  if (typeof amazon_pa_buttons_list !== 'undefined') {
    amazon_pa_buttons_list.forEach(function (element) {
      func_amazon_pa_add_button(element);
    });
  }
}

/**
 * Initialize login widget
 *
 * @returns {void}
 */
function func_amazon_pa_init_login()
{
  if (location.href.indexOf('logout_amazon=1') !== -1) {
    amazon.Login.logout();
  }
  amazon.Login.setClientId(AMAZON_PA_CONST.CID);
}

/**
 * Initialize checkout widgets
 *
 * @returns {Mixed}
 */
function func_amazon_pa_init_checkout() {

  if (typeof OffAmazonPayments === 'undefined') {
    return false;
  }

  if (AMAZON_PA_CONST && AMAZON_PA_CONST.current_checkout_module != 'Amazon_Payments_Advanced') {
    return false;
  }

  if ($('input[name=shippingid]:checked').length === 0) {
    $('button.place-order-button').addClass('inactive');
  }

  $.blockUI.defaults.baseZ = 200000; // default is 1000 and it's too small

  $(document).on('change','input[name=shippingid]', func_amazon_pa_on_change_shipping);
  $('#checkout_form').on('submit', func_amazon_pa_place_order);

  var _addr_object = {
    sellerId: AMAZON_PA_CONST.SID,
    amazonOrderReferenceId: AMAZON_PA_CONST.OREFID,
    design: func_amazon_pa_get_checkout_widget_design(),

    onAddressSelect: function () {
      func_amazon_pa_check_address(AMAZON_PA_CONST.OREFID);
    },
    onError: function (error) {
      if (AMAZON_PA_CONST.MODE === 'test') {
        alert('Amazon AddressBook widget error: code=' + error.getErrorCode() + ' msg=' + error.getErrorMessage());
      }
    }

  };

  if (AMAZON_PA_CONST.prev_method_is_invalid) {
    //customer has to choose another payment. the address cannot be changed
    _addr_object.displayMode = 'Read';
    _addr_object.onAddressSelect = function() {};//do nothing
    amazon_pa_address_selected = true;
    func_amazon_pa_on_change_shipping();//define global shippingid var and check btn states
    $('form[name=shippingsform] :input:not(:button)').prop('disabled', true);
  }

  new OffAmazonPayments.Widgets.AddressBook(_addr_object).bind('addressBookWidgetDiv');

  new OffAmazonPayments.Widgets.Wallet({
    sellerId: AMAZON_PA_CONST.SID,
    amazonOrderReferenceId: AMAZON_PA_CONST.OREFID,
    design: func_amazon_pa_get_checkout_widget_design(),

    onPaymentSelect: function () {
      func_amazon_pa_check_payment(AMAZON_PA_CONST.OREFID);
    },
    onError: function (error) {
      if (AMAZON_PA_CONST.MODE === 'test') {
        alert('Amazon Wallet widget error: code=' + error.getErrorCode() + ' msg=' + error.getErrorMessage());
      }
    }
  }).bind('walletWidgetDiv');

}

/**
 * Adds button widget to DOM
 *
 * @param {string} elmid
 * @returns {Mixed}
 */
function func_amazon_pa_add_button(elmid) {

  if ($('#' + elmid).length <= 0) {
    return false;
  }

  if (typeof OffAmazonPayments === 'undefined') {
    return false;
  }

  var authRequest;
  var addressConsentToken;

  var language = store_language.toLocaleLowerCase() + '-' + store_language.toLocaleUpperCase();

  OffAmazonPayments.Button(elmid, AMAZON_PA_CONST.SID, {
    type:  'PwA',
    color: 'Gold',
    size:  'medium',
    language: language,

    authorization: function() {
      loginOptions = {scope: 'payments:widget payments:shipping_address', popup: true};
      authRequest = amazon.Login.authorize (loginOptions,
        function(response) {
          addressConsentToken = response.access_token;
        });
    },
    onSignIn: function (orderReference) {
      var referenceId = orderReference.getAmazonOrderReferenceId();

      if (!referenceId) {
        alert('Amazon put button widget error: referenceId missing');
      } else {
        window.location = 'amazon_checkout.php'
          + '?amz_pa_gaorid=' + orderReference.getAmazonOrderReferenceId()
          + '&amz_pa_gaact=' + addressConsentToken;
      }
    },
    onError: function (error) {
      if (AMAZON_PA_CONST.MODE === 'test') {
        alert('Amazon put button widget error: code=' + error.getErrorCode() + ' msg=' + error.getErrorMessage());
      }
    }
  });
}

var amazon_pa_place_order_enabled = false;
var amazon_pa_address_selected = false;
var amazon_pa_payment_selected = false;

/**
 * Executed once Amazon login widget is loaded
 *
 * @returns {vaoid}
 */
window.onAmazonLoginReady = function() {
  func_amazon_pa_init_login();
};

/**
 * Executed once Amazon checkout widgets are loaded
 *
 * @returns {vaoid}
 */
window.onAmazonPaymentsReady = function() {
  func_amazon_pa_init_buttons();
  func_amazon_pa_init_checkout();
};

// Init Amazon Widgets.js
$.getScript(AMAZON_PA_CONST.URL, function() {
  $(window).on('resize', func_amazon_pa_init_checkout);
});
